﻿#region Using directives

using System;
using System.Data;
using System.Data.Common;
using System.Collections;
using System.Collections.Generic;

using SecurityFoundation.Entities;
using SecurityFoundation.Data;

#endregion

namespace SecurityFoundation.Data.Bases
{	
	///<summary>
	/// This class is the base class for any <see cref="ObjectAclProviderBase"/> implementation.
	/// It exposes CRUD methods as well as selecting on index, foreign keys and custom stored procedures.
	///</summary>
	public abstract partial class ObjectAclProviderBaseCore : EntityProviderBase<SecurityFoundation.Entities.ObjectAcl, SecurityFoundation.Entities.ObjectAclKey>
	{		
		#region Get from Many To Many Relationship Functions
		#endregion	
		
		#region Delete Methods

		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to delete.</param>
		/// <returns>Returns true if operation suceeded.</returns>
		public override bool Delete(TransactionManager transactionManager, SecurityFoundation.Entities.ObjectAclKey key)
		{
			return Delete(transactionManager, key.ObjectAclid);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="_objectAclid">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public bool Delete(System.Int32 _objectAclid)
		{
			return Delete(null, _objectAclid);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectAclid">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public abstract bool Delete(TransactionManager transactionManager, System.Int32 _objectAclid);		
		
		#endregion Delete Methods
		
		#region Get By Foreign Key Functions
	
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		FK_ObjectACL_ObjectDefinition Description: 
		/// </summary>
		/// <param name="_objectId"></param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		public TList<ObjectAcl> GetByObjectId(System.Guid _objectId)
		{
			int count = -1;
			return GetByObjectId(_objectId, 0,int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		FK_ObjectACL_ObjectDefinition Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectId"></param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		/// <remarks></remarks>
		public TList<ObjectAcl> GetByObjectId(TransactionManager transactionManager, System.Guid _objectId)
		{
			int count = -1;
			return GetByObjectId(transactionManager, _objectId, 0, int.MaxValue, out count);
		}
		
			/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		FK_ObjectACL_ObjectDefinition Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		///  <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		public TList<ObjectAcl> GetByObjectId(TransactionManager transactionManager, System.Guid _objectId, int start, int pageLength)
		{
			int count = -1;
			return GetByObjectId(transactionManager, _objectId, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		fkObjectAclObjectDefinition Description: 
		/// </summary>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="_objectId"></param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		public TList<ObjectAcl> GetByObjectId(System.Guid _objectId, int start, int pageLength)
		{
			int count =  -1;
			return GetByObjectId(null, _objectId, start, pageLength,out count);	
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		fkObjectAclObjectDefinition Description: 
		/// </summary>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="_objectId"></param>
		/// <param name="count">out parameter to get total records for query</param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		public TList<ObjectAcl> GetByObjectId(System.Guid _objectId, int start, int pageLength,out int count)
		{
			return GetByObjectId(null, _objectId, start, pageLength, out count);	
		}
						
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_ObjectACL_ObjectDefinition key.
		///		FK_ObjectACL_ObjectDefinition Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">The total number of records.</param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.ObjectAcl objects.</returns>
		public abstract TList<ObjectAcl> GetByObjectId(TransactionManager transactionManager, System.Guid _objectId, int start, int pageLength, out int count);
		
		#endregion

		#region Get By Index Functions
		
		/// <summary>
		/// 	Gets a row from the DataSource based on its primary key.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to retrieve.</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <returns>Returns an instance of the Entity class.</returns>
		public override SecurityFoundation.Entities.ObjectAcl Get(TransactionManager transactionManager, SecurityFoundation.Entities.ObjectAclKey key, int start, int pageLength)
		{
			return GetByObjectAclid(transactionManager, key.ObjectAclid, start, pageLength);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the primary key PK_ObjectACL index.
		/// </summary>
		/// <param name="_objectAclid"></param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(System.Int32 _objectAclid)
		{
			int count = -1;
			return GetByObjectAclid(null,_objectAclid, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_ObjectACL index.
		/// </summary>
		/// <param name="_objectAclid"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(System.Int32 _objectAclid, int start, int pageLength)
		{
			int count = -1;
			return GetByObjectAclid(null, _objectAclid, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_ObjectACL index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectAclid"></param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(TransactionManager transactionManager, System.Int32 _objectAclid)
		{
			int count = -1;
			return GetByObjectAclid(transactionManager, _objectAclid, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_ObjectACL index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectAclid"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(TransactionManager transactionManager, System.Int32 _objectAclid, int start, int pageLength)
		{
			int count = -1;
			return GetByObjectAclid(transactionManager, _objectAclid, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_ObjectACL index.
		/// </summary>
		/// <param name="_objectAclid"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">out parameter to get total records for query</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(System.Int32 _objectAclid, int start, int pageLength, out int count)
		{
			return GetByObjectAclid(null, _objectAclid, start, pageLength, out count);
		}
		
				
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_ObjectACL index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_objectAclid"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">The total number of records.</param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.ObjectAcl"/> class.</returns>
		public abstract SecurityFoundation.Entities.ObjectAcl GetByObjectAclid(TransactionManager transactionManager, System.Int32 _objectAclid, int start, int pageLength, out int count);
						
		#endregion "Get By Index Functions"
	
		#region Custom Methods
		
		
		#endregion

		#region Helper Functions	
		
		/// <summary>
		/// Fill a TList&lt;ObjectAcl&gt; From a DataReader.
		/// </summary>
		/// <param name="reader">Datareader</param>
		/// <param name="rows">The collection to fill</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">number of rows.</param>
		/// <returns>a <see cref="TList&lt;ObjectAcl&gt;"/></returns>
		public static TList<ObjectAcl> Fill(IDataReader reader, TList<ObjectAcl> rows, int start, int pageLength)
		{
			NetTiersProvider currentProvider = DataRepository.Provider;
            bool useEntityFactory = currentProvider.UseEntityFactory;
            bool enableEntityTracking = currentProvider.EnableEntityTracking;
            LoadPolicy currentLoadPolicy = currentProvider.CurrentLoadPolicy;
			Type entityCreationFactoryType = currentProvider.EntityCreationalFactoryType;
			
			// advance to the starting row
			for (int i = 0; i < start; i++)
			{
				if (!reader.Read())
				return rows; // not enough rows, just return
			}
			for (int i = 0; i < pageLength; i++)
			{
				if (!reader.Read())
					break; // we are done
					
				string key = null;
				
				SecurityFoundation.Entities.ObjectAcl c = null;
				if (useEntityFactory)
				{
					key = new System.Text.StringBuilder("ObjectAcl")
					.Append("|").Append((System.Int32)reader[((int)ObjectAclColumn.ObjectAclid - 1)]).ToString();
					c = EntityManager.LocateOrCreate<ObjectAcl>(
					key.ToString(), // EntityTrackingKey
					"ObjectAcl",  //Creational Type
					entityCreationFactoryType,  //Factory used to create entity
					enableEntityTracking); // Track this entity?
				}
				else
				{
					c = new SecurityFoundation.Entities.ObjectAcl();
				}
				
				if (!enableEntityTracking ||
					c.EntityState == EntityState.Added ||
					(enableEntityTracking &&
					
						(
							(currentLoadPolicy == LoadPolicy.PreserveChanges && c.EntityState == EntityState.Unchanged) ||
							(currentLoadPolicy == LoadPolicy.DiscardChanges && c.EntityState != EntityState.Unchanged)
						)
					))
				{
					c.SuppressEntityEvents = true;
					c.ObjectAclid = (System.Int32)reader[((int)ObjectAclColumn.ObjectAclid - 1)];
					c.ObjectId = (System.Guid)reader[((int)ObjectAclColumn.ObjectId - 1)];
					c.RoleName = (System.String)reader[((int)ObjectAclColumn.RoleName - 1)];
					c.OwnAllowRead = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowRead - 1)];
					c.OwnAllowCreate = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowCreate - 1)];
					c.OwnAllowUpdate = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowUpdate - 1)];
					c.OwnAllowDelete = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowDelete - 1)];
					c.AllAllowRead = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowRead - 1)];
					c.AllAllowCreate = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowCreate - 1)];
					c.AllAllowUpdate = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowUpdate - 1)];
					c.AllAllowDelete = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowDelete - 1)];
					c.Added = (System.DateTime)reader[((int)ObjectAclColumn.Added - 1)];
					c.EntityTrackingKey = key;
					c.AcceptChanges();
					c.SuppressEntityEvents = false;
				}
				rows.Add(c);
			}
		return rows;
		}		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.ObjectAcl"/> object from the <see cref="IDataReader"/>.
		/// </summary>
		/// <param name="reader">The <see cref="IDataReader"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.ObjectAcl"/> object to refresh.</param>
		public static void RefreshEntity(IDataReader reader, SecurityFoundation.Entities.ObjectAcl entity)
		{
			if (!reader.Read()) return;
			
			entity.ObjectAclid = (System.Int32)reader[((int)ObjectAclColumn.ObjectAclid - 1)];
			entity.ObjectId = (System.Guid)reader[((int)ObjectAclColumn.ObjectId - 1)];
			entity.RoleName = (System.String)reader[((int)ObjectAclColumn.RoleName - 1)];
			entity.OwnAllowRead = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowRead - 1)];
			entity.OwnAllowCreate = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowCreate - 1)];
			entity.OwnAllowUpdate = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowUpdate - 1)];
			entity.OwnAllowDelete = (System.Boolean)reader[((int)ObjectAclColumn.OwnAllowDelete - 1)];
			entity.AllAllowRead = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowRead - 1)];
			entity.AllAllowCreate = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowCreate - 1)];
			entity.AllAllowUpdate = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowUpdate - 1)];
			entity.AllAllowDelete = (System.Boolean)reader[((int)ObjectAclColumn.AllAllowDelete - 1)];
			entity.Added = (System.DateTime)reader[((int)ObjectAclColumn.Added - 1)];
			entity.AcceptChanges();
		}
		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.ObjectAcl"/> object from the <see cref="DataSet"/>.
		/// </summary>
		/// <param name="dataSet">The <see cref="DataSet"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.ObjectAcl"/> object.</param>
		public static void RefreshEntity(DataSet dataSet, SecurityFoundation.Entities.ObjectAcl entity)
		{
			DataRow dataRow = dataSet.Tables[0].Rows[0];
			
			entity.ObjectAclid = (System.Int32)dataRow["ObjectACLID"];
			entity.ObjectId = (System.Guid)dataRow["ObjectID"];
			entity.RoleName = (System.String)dataRow["RoleName"];
			entity.OwnAllowRead = (System.Boolean)dataRow["Own_AllowRead"];
			entity.OwnAllowCreate = (System.Boolean)dataRow["Own_AllowCreate"];
			entity.OwnAllowUpdate = (System.Boolean)dataRow["Own_AllowUpdate"];
			entity.OwnAllowDelete = (System.Boolean)dataRow["Own_AllowDelete"];
			entity.AllAllowRead = (System.Boolean)dataRow["All_AllowRead"];
			entity.AllAllowCreate = (System.Boolean)dataRow["All_AllowCreate"];
			entity.AllAllowUpdate = (System.Boolean)dataRow["All_AllowUpdate"];
			entity.AllAllowDelete = (System.Boolean)dataRow["All_AllowDelete"];
			entity.Added = (System.DateTime)dataRow["Added"];
			entity.AcceptChanges();
		}
		#endregion 
		
		#region DeepLoad Methods
		/// <summary>
		/// Deep Loads the <see cref="IEntity"/> object with criteria based of the child 
		/// property collections only N Levels Deep based on the <see cref="DeepLoadType"/>.
		/// </summary>
		/// <remarks>
		/// Use this method with caution as it is possible to DeepLoad with Recursion and traverse an entire object graph.
		/// </remarks>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.ObjectAcl"/> object to load.</param>
		/// <param name="deep">Boolean. A flag that indicates whether to recursively save all Property Collection that are descendants of this instance. If True, saves the complete object graph below this object. If False, saves this object only. </param>
		/// <param name="deepLoadType">DeepLoadType Enumeration to Include/Exclude object property collections from Load.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.ObjectAcl Property Collection Type Array To Include or Exclude from Load</param>
		/// <param name="innerList">A collection of child types for easy access.</param>
	    /// <exception cref="ArgumentNullException">entity or childTypes is null.</exception>
	    /// <exception cref="ArgumentException">deepLoadType has invalid value.</exception>
		public override void DeepLoad(TransactionManager transactionManager, SecurityFoundation.Entities.ObjectAcl entity, bool deep, DeepLoadType deepLoadType, System.Type[] childTypes, DeepSession innerList)
		{
			if(entity == null)
				return;

			#region ObjectIdSource	
			if (CanDeepLoad(entity, "ObjectDefinition|ObjectIdSource", deepLoadType, innerList) 
				&& entity.ObjectIdSource == null)
			{
				object[] pkItems = new object[1];
				pkItems[0] = entity.ObjectId;
				ObjectDefinition tmpEntity = EntityManager.LocateEntity<ObjectDefinition>(EntityLocator.ConstructKeyFromPkItems(typeof(ObjectDefinition), pkItems), DataRepository.Provider.EnableEntityTracking);
				if (tmpEntity != null)
					entity.ObjectIdSource = tmpEntity;
				else
					entity.ObjectIdSource = DataRepository.ObjectDefinitionProvider.GetByObjectId(transactionManager, entity.ObjectId);		
				
				#if NETTIERS_DEBUG
				System.Diagnostics.Debug.WriteLine("- property 'ObjectIdSource' loaded. key " + entity.EntityTrackingKey);
				#endif 
				
				if (deep && entity.ObjectIdSource != null)
				{
					innerList.SkipChildren = true;
					DataRepository.ObjectDefinitionProvider.DeepLoad(transactionManager, entity.ObjectIdSource, deep, deepLoadType, childTypes, innerList);
					innerList.SkipChildren = false;
				}
					
			}
			#endregion ObjectIdSource
			
			//used to hold DeepLoad method delegates and fire after all the local children have been loaded.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
			
			//Fire all DeepLoad Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			deepHandles = null;
		}
		
		#endregion 
		
		#region DeepSave Methods

		/// <summary>
		/// Deep Save the entire object graph of the SecurityFoundation.Entities.ObjectAcl object with criteria based of the child 
		/// Type property array and DeepSaveType.
		/// </summary>
		/// <param name="transactionManager">The transaction manager.</param>
		/// <param name="entity">SecurityFoundation.Entities.ObjectAcl instance</param>
		/// <param name="deepSaveType">DeepSaveType Enumeration to Include/Exclude object property collections from Save.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.ObjectAcl Property Collection Type Array To Include or Exclude from Save</param>
		/// <param name="innerList">A Hashtable of child types for easy access.</param>
		public override bool DeepSave(TransactionManager transactionManager, SecurityFoundation.Entities.ObjectAcl entity, DeepSaveType deepSaveType, System.Type[] childTypes, DeepSession innerList)
		{	
			if (entity == null)
				return false;
							
			#region Composite Parent Properties
			//Save Source Composite Properties, however, don't call deep save on them.  
			//So they only get saved a single level deep.
			
			#region ObjectIdSource
			if (CanDeepSave(entity, "ObjectDefinition|ObjectIdSource", deepSaveType, innerList) 
				&& entity.ObjectIdSource != null)
			{
				DataRepository.ObjectDefinitionProvider.Save(transactionManager, entity.ObjectIdSource);
				entity.ObjectId = entity.ObjectIdSource.ObjectId;
			}
			#endregion 
			#endregion Composite Parent Properties

			// Save Root Entity through Provider
			if (!entity.IsDeleted)
				this.Save(transactionManager, entity);
			
			//used to hold DeepSave method delegates and fire after all the local children have been saved.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
			//Fire all DeepSave Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			
			// Save Root Entity through Provider, if not already saved in delete mode
			if (entity.IsDeleted)
				this.Save(transactionManager, entity);
				

			deepHandles = null;
						
			return true;
		}
		#endregion
	} // end class
	
	#region ObjectAclChildEntityTypes
	
	///<summary>
	/// Enumeration used to expose the different child entity types 
	/// for child properties in <c>SecurityFoundation.Entities.ObjectAcl</c>
	///</summary>
	public enum ObjectAclChildEntityTypes
	{
		
		///<summary>
		/// Composite Property for <c>ObjectDefinition</c> at ObjectIdSource
		///</summary>
		[ChildEntityType(typeof(ObjectDefinition))]
		ObjectDefinition,
		}
	
	#endregion ObjectAclChildEntityTypes
	
	#region ObjectAclFilterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="SqlFilterBuilder&lt;ObjectAclColumn&gt;"/> class
	/// that is used exclusively with a <see cref="ObjectAcl"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class ObjectAclFilterBuilder : SqlFilterBuilder<ObjectAclColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the ObjectAclFilterBuilder class.
		/// </summary>
		public ObjectAclFilterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the ObjectAclFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public ObjectAclFilterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the ObjectAclFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public ObjectAclFilterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion ObjectAclFilterBuilder
	
	#region ObjectAclParameterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="ParameterizedSqlFilterBuilder&lt;ObjectAclColumn&gt;"/> class
	/// that is used exclusively with a <see cref="ObjectAcl"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class ObjectAclParameterBuilder : ParameterizedSqlFilterBuilder<ObjectAclColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the ObjectAclParameterBuilder class.
		/// </summary>
		public ObjectAclParameterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the ObjectAclParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public ObjectAclParameterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the ObjectAclParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public ObjectAclParameterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion ObjectAclParameterBuilder
	
	#region ObjectAclSortBuilder
    
    /// <summary>
    /// A strongly-typed instance of the <see cref="SqlSortBuilder&lt;ObjectAclColumn&gt;"/> class
	/// that is used exclusively with a <see cref="ObjectAcl"/> object.
    /// </summary>
    [CLSCompliant(true)]
    public class ObjectAclSortBuilder : SqlSortBuilder<ObjectAclColumn>
    {
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the ObjectAclSqlSortBuilder class.
		/// </summary>
		public ObjectAclSortBuilder() : base() { }

		#endregion Constructors

    }    
    #endregion ObjectAclSortBuilder
	
} // end namespace
