﻿#region Using directives

using System;
using System.Data;
using System.Data.Common;
using System.Collections;
using System.Collections.Generic;

using SecurityFoundation.Entities;
using SecurityFoundation.Data;

#endregion

namespace SecurityFoundation.Data.Bases
{	
	///<summary>
	/// This class is the base class for any <see cref="RealmProviderBase"/> implementation.
	/// It exposes CRUD methods as well as selecting on index, foreign keys and custom stored procedures.
	///</summary>
	public abstract partial class RealmProviderBaseCore : EntityProviderBase<SecurityFoundation.Entities.Realm, SecurityFoundation.Entities.RealmKey>
	{		
		#region Get from Many To Many Relationship Functions
		#endregion	
		
		#region Delete Methods

		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to delete.</param>
		/// <returns>Returns true if operation suceeded.</returns>
		public override bool Delete(TransactionManager transactionManager, SecurityFoundation.Entities.RealmKey key)
		{
			return Delete(transactionManager, key.RealmId);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="_realmId">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public bool Delete(System.Guid _realmId)
		{
			return Delete(null, _realmId);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_realmId">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public abstract bool Delete(TransactionManager transactionManager, System.Guid _realmId);		
		
		#endregion Delete Methods
		
		#region Get By Foreign Key Functions
		#endregion

		#region Get By Index Functions
		
		/// <summary>
		/// 	Gets a row from the DataSource based on its primary key.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to retrieve.</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <returns>Returns an instance of the Entity class.</returns>
		public override SecurityFoundation.Entities.Realm Get(TransactionManager transactionManager, SecurityFoundation.Entities.RealmKey key, int start, int pageLength)
		{
			return GetByRealmId(transactionManager, key.RealmId, start, pageLength);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the primary key PK_Realm index.
		/// </summary>
		/// <param name="_realmId"></param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public SecurityFoundation.Entities.Realm GetByRealmId(System.Guid _realmId)
		{
			int count = -1;
			return GetByRealmId(null,_realmId, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_Realm index.
		/// </summary>
		/// <param name="_realmId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public SecurityFoundation.Entities.Realm GetByRealmId(System.Guid _realmId, int start, int pageLength)
		{
			int count = -1;
			return GetByRealmId(null, _realmId, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_Realm index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_realmId"></param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public SecurityFoundation.Entities.Realm GetByRealmId(TransactionManager transactionManager, System.Guid _realmId)
		{
			int count = -1;
			return GetByRealmId(transactionManager, _realmId, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_Realm index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_realmId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public SecurityFoundation.Entities.Realm GetByRealmId(TransactionManager transactionManager, System.Guid _realmId, int start, int pageLength)
		{
			int count = -1;
			return GetByRealmId(transactionManager, _realmId, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_Realm index.
		/// </summary>
		/// <param name="_realmId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">out parameter to get total records for query</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public SecurityFoundation.Entities.Realm GetByRealmId(System.Guid _realmId, int start, int pageLength, out int count)
		{
			return GetByRealmId(null, _realmId, start, pageLength, out count);
		}
		
				
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_Realm index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_realmId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">The total number of records.</param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.Realm"/> class.</returns>
		public abstract SecurityFoundation.Entities.Realm GetByRealmId(TransactionManager transactionManager, System.Guid _realmId, int start, int pageLength, out int count);
						
		#endregion "Get By Index Functions"
	
		#region Custom Methods
		
		
		#endregion

		#region Helper Functions	
		
		/// <summary>
		/// Fill a TList&lt;Realm&gt; From a DataReader.
		/// </summary>
		/// <param name="reader">Datareader</param>
		/// <param name="rows">The collection to fill</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">number of rows.</param>
		/// <returns>a <see cref="TList&lt;Realm&gt;"/></returns>
		public static TList<Realm> Fill(IDataReader reader, TList<Realm> rows, int start, int pageLength)
		{
			NetTiersProvider currentProvider = DataRepository.Provider;
            bool useEntityFactory = currentProvider.UseEntityFactory;
            bool enableEntityTracking = currentProvider.EnableEntityTracking;
            LoadPolicy currentLoadPolicy = currentProvider.CurrentLoadPolicy;
			Type entityCreationFactoryType = currentProvider.EntityCreationalFactoryType;
			
			// advance to the starting row
			for (int i = 0; i < start; i++)
			{
				if (!reader.Read())
				return rows; // not enough rows, just return
			}
			for (int i = 0; i < pageLength; i++)
			{
				if (!reader.Read())
					break; // we are done
					
				string key = null;
				
				SecurityFoundation.Entities.Realm c = null;
				if (useEntityFactory)
				{
					key = new System.Text.StringBuilder("Realm")
					.Append("|").Append((System.Guid)reader[((int)RealmColumn.RealmId - 1)]).ToString();
					c = EntityManager.LocateOrCreate<Realm>(
					key.ToString(), // EntityTrackingKey
					"Realm",  //Creational Type
					entityCreationFactoryType,  //Factory used to create entity
					enableEntityTracking); // Track this entity?
				}
				else
				{
					c = new SecurityFoundation.Entities.Realm();
				}
				
				if (!enableEntityTracking ||
					c.EntityState == EntityState.Added ||
					(enableEntityTracking &&
					
						(
							(currentLoadPolicy == LoadPolicy.PreserveChanges && c.EntityState == EntityState.Unchanged) ||
							(currentLoadPolicy == LoadPolicy.DiscardChanges && c.EntityState != EntityState.Unchanged)
						)
					))
				{
					c.SuppressEntityEvents = true;
					c.RealmId = (System.Guid)reader[((int)RealmColumn.RealmId - 1)];
					c.OriginalRealmId = c.RealmId;
					c.Realm = (System.String)reader[((int)RealmColumn.Realm - 1)];
					c.AuthenticationProviderName = (reader.IsDBNull(((int)RealmColumn.AuthenticationProviderName - 1)))?null:(System.String)reader[((int)RealmColumn.AuthenticationProviderName - 1)];
					c.RoleProviderName = (reader.IsDBNull(((int)RealmColumn.RoleProviderName - 1)))?null:(System.String)reader[((int)RealmColumn.RoleProviderName - 1)];
					c.TokenLife = (System.Int32)reader[((int)RealmColumn.TokenLife - 1)];
					c.AdminApplication = (reader.IsDBNull(((int)RealmColumn.AdminApplication - 1)))?null:(System.Guid?)reader[((int)RealmColumn.AdminApplication - 1)];
					c.EntityTrackingKey = key;
					c.AcceptChanges();
					c.SuppressEntityEvents = false;
				}
				rows.Add(c);
			}
		return rows;
		}		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.Realm"/> object from the <see cref="IDataReader"/>.
		/// </summary>
		/// <param name="reader">The <see cref="IDataReader"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.Realm"/> object to refresh.</param>
		public static void RefreshEntity(IDataReader reader, SecurityFoundation.Entities.Realm entity)
		{
			if (!reader.Read()) return;
			
			entity.RealmId = (System.Guid)reader[((int)RealmColumn.RealmId - 1)];
			entity.OriginalRealmId = (System.Guid)reader["RealmID"];
			entity.Realm = (System.String)reader[((int)RealmColumn.Realm - 1)];
			entity.AuthenticationProviderName = (reader.IsDBNull(((int)RealmColumn.AuthenticationProviderName - 1)))?null:(System.String)reader[((int)RealmColumn.AuthenticationProviderName - 1)];
			entity.RoleProviderName = (reader.IsDBNull(((int)RealmColumn.RoleProviderName - 1)))?null:(System.String)reader[((int)RealmColumn.RoleProviderName - 1)];
			entity.TokenLife = (System.Int32)reader[((int)RealmColumn.TokenLife - 1)];
			entity.AdminApplication = (reader.IsDBNull(((int)RealmColumn.AdminApplication - 1)))?null:(System.Guid?)reader[((int)RealmColumn.AdminApplication - 1)];
			entity.AcceptChanges();
		}
		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.Realm"/> object from the <see cref="DataSet"/>.
		/// </summary>
		/// <param name="dataSet">The <see cref="DataSet"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.Realm"/> object.</param>
		public static void RefreshEntity(DataSet dataSet, SecurityFoundation.Entities.Realm entity)
		{
			DataRow dataRow = dataSet.Tables[0].Rows[0];
			
			entity.RealmId = (System.Guid)dataRow["RealmID"];
			entity.OriginalRealmId = (System.Guid)dataRow["RealmID"];
			entity.Realm = (System.String)dataRow["Realm"];
			entity.AuthenticationProviderName = Convert.IsDBNull(dataRow["AuthenticationProviderName"]) ? null : (System.String)dataRow["AuthenticationProviderName"];
			entity.RoleProviderName = Convert.IsDBNull(dataRow["RoleProviderName"]) ? null : (System.String)dataRow["RoleProviderName"];
			entity.TokenLife = (System.Int32)dataRow["TokenLife"];
			entity.AdminApplication = Convert.IsDBNull(dataRow["AdminApplication"]) ? null : (System.Guid?)dataRow["AdminApplication"];
			entity.AcceptChanges();
		}
		#endregion 
		
		#region DeepLoad Methods
		/// <summary>
		/// Deep Loads the <see cref="IEntity"/> object with criteria based of the child 
		/// property collections only N Levels Deep based on the <see cref="DeepLoadType"/>.
		/// </summary>
		/// <remarks>
		/// Use this method with caution as it is possible to DeepLoad with Recursion and traverse an entire object graph.
		/// </remarks>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.Realm"/> object to load.</param>
		/// <param name="deep">Boolean. A flag that indicates whether to recursively save all Property Collection that are descendants of this instance. If True, saves the complete object graph below this object. If False, saves this object only. </param>
		/// <param name="deepLoadType">DeepLoadType Enumeration to Include/Exclude object property collections from Load.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.Realm Property Collection Type Array To Include or Exclude from Load</param>
		/// <param name="innerList">A collection of child types for easy access.</param>
	    /// <exception cref="ArgumentNullException">entity or childTypes is null.</exception>
	    /// <exception cref="ArgumentException">deepLoadType has invalid value.</exception>
		public override void DeepLoad(TransactionManager transactionManager, SecurityFoundation.Entities.Realm entity, bool deep, DeepLoadType deepLoadType, System.Type[] childTypes, DeepSession innerList)
		{
			if(entity == null)
				return;
			
			//used to hold DeepLoad method delegates and fire after all the local children have been loaded.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
			// Deep load child collections  - Call GetByRealmId methods when available
			
			#region ApplicationDefinationCollection
			//Relationship Type One : Many
			if (CanDeepLoad(entity, "List<ApplicationDefination>|ApplicationDefinationCollection", deepLoadType, innerList)) 
			{
				#if NETTIERS_DEBUG
				System.Diagnostics.Debug.WriteLine("- property 'ApplicationDefinationCollection' loaded. key " + entity.EntityTrackingKey);
				#endif 

				entity.ApplicationDefinationCollection = DataRepository.ApplicationDefinationProvider.GetByRealm(transactionManager, entity.RealmId);

				if (deep && entity.ApplicationDefinationCollection.Count > 0)
				{
					deepHandles.Add("ApplicationDefinationCollection",
						new KeyValuePair<Delegate, object>((DeepLoadHandle<ApplicationDefination>) DataRepository.ApplicationDefinationProvider.DeepLoad,
						new object[] { transactionManager, entity.ApplicationDefinationCollection, deep, deepLoadType, childTypes, innerList }
					));
				}
			}		
			#endregion 
			
			
			#region SecurityTokenCollection
			//Relationship Type One : Many
			if (CanDeepLoad(entity, "List<SecurityToken>|SecurityTokenCollection", deepLoadType, innerList)) 
			{
				#if NETTIERS_DEBUG
				System.Diagnostics.Debug.WriteLine("- property 'SecurityTokenCollection' loaded. key " + entity.EntityTrackingKey);
				#endif 

				entity.SecurityTokenCollection = DataRepository.SecurityTokenProvider.GetByRealm(transactionManager, entity.RealmId);

				if (deep && entity.SecurityTokenCollection.Count > 0)
				{
					deepHandles.Add("SecurityTokenCollection",
						new KeyValuePair<Delegate, object>((DeepLoadHandle<SecurityToken>) DataRepository.SecurityTokenProvider.DeepLoad,
						new object[] { transactionManager, entity.SecurityTokenCollection, deep, deepLoadType, childTypes, innerList }
					));
				}
			}		
			#endregion 
			
			
			//Fire all DeepLoad Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			deepHandles = null;
		}
		
		#endregion 
		
		#region DeepSave Methods

		/// <summary>
		/// Deep Save the entire object graph of the SecurityFoundation.Entities.Realm object with criteria based of the child 
		/// Type property array and DeepSaveType.
		/// </summary>
		/// <param name="transactionManager">The transaction manager.</param>
		/// <param name="entity">SecurityFoundation.Entities.Realm instance</param>
		/// <param name="deepSaveType">DeepSaveType Enumeration to Include/Exclude object property collections from Save.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.Realm Property Collection Type Array To Include or Exclude from Save</param>
		/// <param name="innerList">A Hashtable of child types for easy access.</param>
		public override bool DeepSave(TransactionManager transactionManager, SecurityFoundation.Entities.Realm entity, DeepSaveType deepSaveType, System.Type[] childTypes, DeepSession innerList)
		{	
			if (entity == null)
				return false;
							
			#region Composite Parent Properties
			//Save Source Composite Properties, however, don't call deep save on them.  
			//So they only get saved a single level deep.
			#endregion Composite Parent Properties

			// Save Root Entity through Provider
			if (!entity.IsDeleted)
				this.Save(transactionManager, entity);
			
			//used to hold DeepSave method delegates and fire after all the local children have been saved.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
	
			#region List<ApplicationDefination>
				if (CanDeepSave(entity.ApplicationDefinationCollection, "List<ApplicationDefination>|ApplicationDefinationCollection", deepSaveType, innerList)) 
				{	
					// update each child parent id with the real parent id (mostly used on insert)
					foreach(ApplicationDefination child in entity.ApplicationDefinationCollection)
					{
						if(child.RealmSource != null)
						{
							child.Realm = child.RealmSource.RealmId;
						}
						else
						{
							child.Realm = entity.RealmId;
						}

					}

					if (entity.ApplicationDefinationCollection.Count > 0 || entity.ApplicationDefinationCollection.DeletedItems.Count > 0)
					{
						//DataRepository.ApplicationDefinationProvider.Save(transactionManager, entity.ApplicationDefinationCollection);
						
						deepHandles.Add("ApplicationDefinationCollection",
						new KeyValuePair<Delegate, object>((DeepSaveHandle< ApplicationDefination >) DataRepository.ApplicationDefinationProvider.DeepSave,
							new object[] { transactionManager, entity.ApplicationDefinationCollection, deepSaveType, childTypes, innerList }
						));
					}
				} 
			#endregion 
				
	
			#region List<SecurityToken>
				if (CanDeepSave(entity.SecurityTokenCollection, "List<SecurityToken>|SecurityTokenCollection", deepSaveType, innerList)) 
				{	
					// update each child parent id with the real parent id (mostly used on insert)
					foreach(SecurityToken child in entity.SecurityTokenCollection)
					{
						if(child.RealmSource != null)
						{
							child.Realm = child.RealmSource.RealmId;
						}
						else
						{
							child.Realm = entity.RealmId;
						}

					}

					if (entity.SecurityTokenCollection.Count > 0 || entity.SecurityTokenCollection.DeletedItems.Count > 0)
					{
						//DataRepository.SecurityTokenProvider.Save(transactionManager, entity.SecurityTokenCollection);
						
						deepHandles.Add("SecurityTokenCollection",
						new KeyValuePair<Delegate, object>((DeepSaveHandle< SecurityToken >) DataRepository.SecurityTokenProvider.DeepSave,
							new object[] { transactionManager, entity.SecurityTokenCollection, deepSaveType, childTypes, innerList }
						));
					}
				} 
			#endregion 
				
			//Fire all DeepSave Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			
			// Save Root Entity through Provider, if not already saved in delete mode
			if (entity.IsDeleted)
				this.Save(transactionManager, entity);
				

			deepHandles = null;
						
			return true;
		}
		#endregion
	} // end class
	
	#region RealmChildEntityTypes
	
	///<summary>
	/// Enumeration used to expose the different child entity types 
	/// for child properties in <c>SecurityFoundation.Entities.Realm</c>
	///</summary>
	public enum RealmChildEntityTypes
	{

		///<summary>
		/// Collection of <c>Realm</c> as OneToMany for ApplicationDefinationCollection
		///</summary>
		[ChildEntityType(typeof(TList<ApplicationDefination>))]
		ApplicationDefinationCollection,

		///<summary>
		/// Collection of <c>Realm</c> as OneToMany for SecurityTokenCollection
		///</summary>
		[ChildEntityType(typeof(TList<SecurityToken>))]
		SecurityTokenCollection,
	}
	
	#endregion RealmChildEntityTypes
	
	#region RealmFilterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="SqlFilterBuilder&lt;RealmColumn&gt;"/> class
	/// that is used exclusively with a <see cref="Realm"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class RealmFilterBuilder : SqlFilterBuilder<RealmColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RealmFilterBuilder class.
		/// </summary>
		public RealmFilterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the RealmFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public RealmFilterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the RealmFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public RealmFilterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion RealmFilterBuilder
	
	#region RealmParameterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="ParameterizedSqlFilterBuilder&lt;RealmColumn&gt;"/> class
	/// that is used exclusively with a <see cref="Realm"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class RealmParameterBuilder : ParameterizedSqlFilterBuilder<RealmColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RealmParameterBuilder class.
		/// </summary>
		public RealmParameterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the RealmParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public RealmParameterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the RealmParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public RealmParameterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion RealmParameterBuilder
	
	#region RealmSortBuilder
    
    /// <summary>
    /// A strongly-typed instance of the <see cref="SqlSortBuilder&lt;RealmColumn&gt;"/> class
	/// that is used exclusively with a <see cref="Realm"/> object.
    /// </summary>
    [CLSCompliant(true)]
    public class RealmSortBuilder : SqlSortBuilder<RealmColumn>
    {
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RealmSqlSortBuilder class.
		/// </summary>
		public RealmSortBuilder() : base() { }

		#endregion Constructors

    }    
    #endregion RealmSortBuilder
	
} // end namespace
