﻿#region Using directives

using System;
using System.Data;
using System.Data.Common;
using System.Collections;
using System.Collections.Generic;

using SecurityFoundation.Entities;
using SecurityFoundation.Data;

#endregion

namespace SecurityFoundation.Data.Bases
{	
	///<summary>
	/// This class is the base class for any <see cref="UserProfileDataProviderBase"/> implementation.
	/// It exposes CRUD methods as well as selecting on index, foreign keys and custom stored procedures.
	///</summary>
	public abstract partial class UserProfileDataProviderBaseCore : EntityProviderBase<SecurityFoundation.Entities.UserProfileData, SecurityFoundation.Entities.UserProfileDataKey>
	{		
		#region Get from Many To Many Relationship Functions
		#endregion	
		
		#region Delete Methods

		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to delete.</param>
		/// <returns>Returns true if operation suceeded.</returns>
		public override bool Delete(TransactionManager transactionManager, SecurityFoundation.Entities.UserProfileDataKey key)
		{
			return Delete(transactionManager, key.ProfileDataId);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="_profileDataId">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public bool Delete(System.Int32 _profileDataId)
		{
			return Delete(null, _profileDataId);
		}
		
		/// <summary>
		/// 	Deletes a row from the DataSource.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_profileDataId">. Primary Key.</param>
		/// <remarks>Deletes based on primary key(s).</remarks>
		/// <returns>Returns true if operation suceeded.</returns>
		public abstract bool Delete(TransactionManager transactionManager, System.Int32 _profileDataId);		
		
		#endregion Delete Methods
		
		#region Get By Foreign Key Functions
	
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		FK_UserProfileData_ApplicationDefination Description: 
		/// </summary>
		/// <param name="_applicationKey"></param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		public TList<UserProfileData> GetByApplicationKey(System.Guid _applicationKey)
		{
			int count = -1;
			return GetByApplicationKey(_applicationKey, 0,int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		FK_UserProfileData_ApplicationDefination Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_applicationKey"></param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		/// <remarks></remarks>
		public TList<UserProfileData> GetByApplicationKey(TransactionManager transactionManager, System.Guid _applicationKey)
		{
			int count = -1;
			return GetByApplicationKey(transactionManager, _applicationKey, 0, int.MaxValue, out count);
		}
		
			/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		FK_UserProfileData_ApplicationDefination Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_applicationKey"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		///  <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		public TList<UserProfileData> GetByApplicationKey(TransactionManager transactionManager, System.Guid _applicationKey, int start, int pageLength)
		{
			int count = -1;
			return GetByApplicationKey(transactionManager, _applicationKey, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		fkUserProfileDataApplicationDefination Description: 
		/// </summary>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="_applicationKey"></param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		public TList<UserProfileData> GetByApplicationKey(System.Guid _applicationKey, int start, int pageLength)
		{
			int count =  -1;
			return GetByApplicationKey(null, _applicationKey, start, pageLength,out count);	
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		fkUserProfileDataApplicationDefination Description: 
		/// </summary>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="_applicationKey"></param>
		/// <param name="count">out parameter to get total records for query</param>
		/// <remarks></remarks>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		public TList<UserProfileData> GetByApplicationKey(System.Guid _applicationKey, int start, int pageLength,out int count)
		{
			return GetByApplicationKey(null, _applicationKey, start, pageLength, out count);	
		}
						
		/// <summary>
		/// 	Gets rows from the datasource based on the FK_UserProfileData_ApplicationDefination key.
		///		FK_UserProfileData_ApplicationDefination Description: 
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_applicationKey"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">The total number of records.</param>
		/// <returns>Returns a typed collection of SecurityFoundation.Entities.UserProfileData objects.</returns>
		public abstract TList<UserProfileData> GetByApplicationKey(TransactionManager transactionManager, System.Guid _applicationKey, int start, int pageLength, out int count);
		
		#endregion

		#region Get By Index Functions
		
		/// <summary>
		/// 	Gets a row from the DataSource based on its primary key.
		/// </summary>
		/// <param name="transactionManager">A <see cref="TransactionManager"/> object.</param>
		/// <param name="key">The unique identifier of the row to retrieve.</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <returns>Returns an instance of the Entity class.</returns>
		public override SecurityFoundation.Entities.UserProfileData Get(TransactionManager transactionManager, SecurityFoundation.Entities.UserProfileDataKey key, int start, int pageLength)
		{
			return GetByProfileDataId(transactionManager, key.ProfileDataId, start, pageLength);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the primary key PK_UserProfileData index.
		/// </summary>
		/// <param name="_profileDataId"></param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public SecurityFoundation.Entities.UserProfileData GetByProfileDataId(System.Int32 _profileDataId)
		{
			int count = -1;
			return GetByProfileDataId(null,_profileDataId, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_UserProfileData index.
		/// </summary>
		/// <param name="_profileDataId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public SecurityFoundation.Entities.UserProfileData GetByProfileDataId(System.Int32 _profileDataId, int start, int pageLength)
		{
			int count = -1;
			return GetByProfileDataId(null, _profileDataId, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_UserProfileData index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_profileDataId"></param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public SecurityFoundation.Entities.UserProfileData GetByProfileDataId(TransactionManager transactionManager, System.Int32 _profileDataId)
		{
			int count = -1;
			return GetByProfileDataId(transactionManager, _profileDataId, 0, int.MaxValue, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_UserProfileData index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_profileDataId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public SecurityFoundation.Entities.UserProfileData GetByProfileDataId(TransactionManager transactionManager, System.Int32 _profileDataId, int start, int pageLength)
		{
			int count = -1;
			return GetByProfileDataId(transactionManager, _profileDataId, start, pageLength, out count);
		}
		
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_UserProfileData index.
		/// </summary>
		/// <param name="_profileDataId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">out parameter to get total records for query</param>
		/// <remarks></remarks>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public SecurityFoundation.Entities.UserProfileData GetByProfileDataId(System.Int32 _profileDataId, int start, int pageLength, out int count)
		{
			return GetByProfileDataId(null, _profileDataId, start, pageLength, out count);
		}
		
				
		/// <summary>
		/// 	Gets rows from the datasource based on the PK_UserProfileData index.
		/// </summary>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="_profileDataId"></param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="count">The total number of records.</param>
		/// <returns>Returns an instance of the <see cref="SecurityFoundation.Entities.UserProfileData"/> class.</returns>
		public abstract SecurityFoundation.Entities.UserProfileData GetByProfileDataId(TransactionManager transactionManager, System.Int32 _profileDataId, int start, int pageLength, out int count);
						
		#endregion "Get By Index Functions"
	
		#region Custom Methods
		
		
		#region usp_UserProfileData_GetByApplicationUser 
		
		/// <summary>
		///	This method wrap the 'usp_UserProfileData_GetByApplicationUser' stored procedure. 
		/// </summary>
		/// <param name="app"> A <c>System.Guid?</c> instance.</param>
		/// <param name="userName"> A <c>System.String</c> instance.</param>
		/// <remark>This method is generate from a stored procedure.</remark>
		/// <returns>A <see cref="TList&lt;UserProfileData&gt;"/> instance.</returns>
		public TList<UserProfileData> GetByApplicationUser(System.Guid? app, System.String userName)
		{
			return GetByApplicationUser(null, 0, int.MaxValue , app, userName);
		}
		
		/// <summary>
		///	This method wrap the 'usp_UserProfileData_GetByApplicationUser' stored procedure. 
		/// </summary>
		/// <param name="app"> A <c>System.Guid?</c> instance.</param>
		/// <param name="userName"> A <c>System.String</c> instance.</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <remark>This method is generate from a stored procedure.</remark>
		/// <returns>A <see cref="TList&lt;UserProfileData&gt;"/> instance.</returns>
		public TList<UserProfileData> GetByApplicationUser(int start, int pageLength, System.Guid? app, System.String userName)
		{
			return GetByApplicationUser(null, start, pageLength , app, userName);
		}
				
		/// <summary>
		///	This method wrap the 'usp_UserProfileData_GetByApplicationUser' stored procedure. 
		/// </summary>
		/// <param name="app"> A <c>System.Guid?</c> instance.</param>
		/// <param name="userName"> A <c>System.String</c> instance.</param>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <remark>This method is generate from a stored procedure.</remark>
		/// <returns>A <see cref="TList&lt;UserProfileData&gt;"/> instance.</returns>
		public TList<UserProfileData> GetByApplicationUser(TransactionManager transactionManager, System.Guid? app, System.String userName)
		{
			return GetByApplicationUser(transactionManager, 0, int.MaxValue , app, userName);
		}
		
		/// <summary>
		///	This method wrap the 'usp_UserProfileData_GetByApplicationUser' stored procedure. 
		/// </summary>
		/// <param name="app"> A <c>System.Guid?</c> instance.</param>
		/// <param name="userName"> A <c>System.String</c> instance.</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">Number of rows to return.</param>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <remark>This method is generate from a stored procedure.</remark>
		/// <returns>A <see cref="TList&lt;UserProfileData&gt;"/> instance.</returns>
		public abstract TList<UserProfileData> GetByApplicationUser(TransactionManager transactionManager, int start, int pageLength , System.Guid? app, System.String userName);
		
		#endregion
		
		#endregion

		#region Helper Functions	
		
		/// <summary>
		/// Fill a TList&lt;UserProfileData&gt; From a DataReader.
		/// </summary>
		/// <param name="reader">Datareader</param>
		/// <param name="rows">The collection to fill</param>
		/// <param name="start">Row number at which to start reading, the first row is 0.</param>
		/// <param name="pageLength">number of rows.</param>
		/// <returns>a <see cref="TList&lt;UserProfileData&gt;"/></returns>
		public static TList<UserProfileData> Fill(IDataReader reader, TList<UserProfileData> rows, int start, int pageLength)
		{
			NetTiersProvider currentProvider = DataRepository.Provider;
            bool useEntityFactory = currentProvider.UseEntityFactory;
            bool enableEntityTracking = currentProvider.EnableEntityTracking;
            LoadPolicy currentLoadPolicy = currentProvider.CurrentLoadPolicy;
			Type entityCreationFactoryType = currentProvider.EntityCreationalFactoryType;
			
			// advance to the starting row
			for (int i = 0; i < start; i++)
			{
				if (!reader.Read())
				return rows; // not enough rows, just return
			}
			for (int i = 0; i < pageLength; i++)
			{
				if (!reader.Read())
					break; // we are done
					
				string key = null;
				
				SecurityFoundation.Entities.UserProfileData c = null;
				if (useEntityFactory)
				{
					key = new System.Text.StringBuilder("UserProfileData")
					.Append("|").Append((System.Int32)reader[((int)UserProfileDataColumn.ProfileDataId - 1)]).ToString();
					c = EntityManager.LocateOrCreate<UserProfileData>(
					key.ToString(), // EntityTrackingKey
					"UserProfileData",  //Creational Type
					entityCreationFactoryType,  //Factory used to create entity
					enableEntityTracking); // Track this entity?
				}
				else
				{
					c = new SecurityFoundation.Entities.UserProfileData();
				}
				
				if (!enableEntityTracking ||
					c.EntityState == EntityState.Added ||
					(enableEntityTracking &&
					
						(
							(currentLoadPolicy == LoadPolicy.PreserveChanges && c.EntityState == EntityState.Unchanged) ||
							(currentLoadPolicy == LoadPolicy.DiscardChanges && c.EntityState != EntityState.Unchanged)
						)
					))
				{
					c.SuppressEntityEvents = true;
					c.ProfileDataId = (System.Int32)reader[((int)UserProfileDataColumn.ProfileDataId - 1)];
					c.UserName = (System.String)reader[((int)UserProfileDataColumn.UserName - 1)];
					c.ApplicationKey = (System.Guid)reader[((int)UserProfileDataColumn.ApplicationKey - 1)];
					c.ProfileData = (System.String)reader[((int)UserProfileDataColumn.ProfileData - 1)];
					c.LastUpdated = (System.DateTime)reader[((int)UserProfileDataColumn.LastUpdated - 1)];
					c.EntityTrackingKey = key;
					c.AcceptChanges();
					c.SuppressEntityEvents = false;
				}
				rows.Add(c);
			}
		return rows;
		}		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.UserProfileData"/> object from the <see cref="IDataReader"/>.
		/// </summary>
		/// <param name="reader">The <see cref="IDataReader"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.UserProfileData"/> object to refresh.</param>
		public static void RefreshEntity(IDataReader reader, SecurityFoundation.Entities.UserProfileData entity)
		{
			if (!reader.Read()) return;
			
			entity.ProfileDataId = (System.Int32)reader[((int)UserProfileDataColumn.ProfileDataId - 1)];
			entity.UserName = (System.String)reader[((int)UserProfileDataColumn.UserName - 1)];
			entity.ApplicationKey = (System.Guid)reader[((int)UserProfileDataColumn.ApplicationKey - 1)];
			entity.ProfileData = (System.String)reader[((int)UserProfileDataColumn.ProfileData - 1)];
			entity.LastUpdated = (System.DateTime)reader[((int)UserProfileDataColumn.LastUpdated - 1)];
			entity.AcceptChanges();
		}
		
		/// <summary>
		/// Refreshes the <see cref="SecurityFoundation.Entities.UserProfileData"/> object from the <see cref="DataSet"/>.
		/// </summary>
		/// <param name="dataSet">The <see cref="DataSet"/> to read from.</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.UserProfileData"/> object.</param>
		public static void RefreshEntity(DataSet dataSet, SecurityFoundation.Entities.UserProfileData entity)
		{
			DataRow dataRow = dataSet.Tables[0].Rows[0];
			
			entity.ProfileDataId = (System.Int32)dataRow["ProfileDataID"];
			entity.UserName = (System.String)dataRow["UserName"];
			entity.ApplicationKey = (System.Guid)dataRow["ApplicationKey"];
			entity.ProfileData = (System.String)dataRow["ProfileData"];
			entity.LastUpdated = (System.DateTime)dataRow["LastUpdated"];
			entity.AcceptChanges();
		}
		#endregion 
		
		#region DeepLoad Methods
		/// <summary>
		/// Deep Loads the <see cref="IEntity"/> object with criteria based of the child 
		/// property collections only N Levels Deep based on the <see cref="DeepLoadType"/>.
		/// </summary>
		/// <remarks>
		/// Use this method with caution as it is possible to DeepLoad with Recursion and traverse an entire object graph.
		/// </remarks>
		/// <param name="transactionManager"><see cref="TransactionManager"/> object</param>
		/// <param name="entity">The <see cref="SecurityFoundation.Entities.UserProfileData"/> object to load.</param>
		/// <param name="deep">Boolean. A flag that indicates whether to recursively save all Property Collection that are descendants of this instance. If True, saves the complete object graph below this object. If False, saves this object only. </param>
		/// <param name="deepLoadType">DeepLoadType Enumeration to Include/Exclude object property collections from Load.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.UserProfileData Property Collection Type Array To Include or Exclude from Load</param>
		/// <param name="innerList">A collection of child types for easy access.</param>
	    /// <exception cref="ArgumentNullException">entity or childTypes is null.</exception>
	    /// <exception cref="ArgumentException">deepLoadType has invalid value.</exception>
		public override void DeepLoad(TransactionManager transactionManager, SecurityFoundation.Entities.UserProfileData entity, bool deep, DeepLoadType deepLoadType, System.Type[] childTypes, DeepSession innerList)
		{
			if(entity == null)
				return;

			#region ApplicationKeySource	
			if (CanDeepLoad(entity, "ApplicationDefination|ApplicationKeySource", deepLoadType, innerList) 
				&& entity.ApplicationKeySource == null)
			{
				object[] pkItems = new object[1];
				pkItems[0] = entity.ApplicationKey;
				ApplicationDefination tmpEntity = EntityManager.LocateEntity<ApplicationDefination>(EntityLocator.ConstructKeyFromPkItems(typeof(ApplicationDefination), pkItems), DataRepository.Provider.EnableEntityTracking);
				if (tmpEntity != null)
					entity.ApplicationKeySource = tmpEntity;
				else
					entity.ApplicationKeySource = DataRepository.ApplicationDefinationProvider.GetByApplicationKey(transactionManager, entity.ApplicationKey);		
				
				#if NETTIERS_DEBUG
				System.Diagnostics.Debug.WriteLine("- property 'ApplicationKeySource' loaded. key " + entity.EntityTrackingKey);
				#endif 
				
				if (deep && entity.ApplicationKeySource != null)
				{
					innerList.SkipChildren = true;
					DataRepository.ApplicationDefinationProvider.DeepLoad(transactionManager, entity.ApplicationKeySource, deep, deepLoadType, childTypes, innerList);
					innerList.SkipChildren = false;
				}
					
			}
			#endregion ApplicationKeySource
			
			//used to hold DeepLoad method delegates and fire after all the local children have been loaded.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
			
			//Fire all DeepLoad Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			deepHandles = null;
		}
		
		#endregion 
		
		#region DeepSave Methods

		/// <summary>
		/// Deep Save the entire object graph of the SecurityFoundation.Entities.UserProfileData object with criteria based of the child 
		/// Type property array and DeepSaveType.
		/// </summary>
		/// <param name="transactionManager">The transaction manager.</param>
		/// <param name="entity">SecurityFoundation.Entities.UserProfileData instance</param>
		/// <param name="deepSaveType">DeepSaveType Enumeration to Include/Exclude object property collections from Save.</param>
		/// <param name="childTypes">SecurityFoundation.Entities.UserProfileData Property Collection Type Array To Include or Exclude from Save</param>
		/// <param name="innerList">A Hashtable of child types for easy access.</param>
		public override bool DeepSave(TransactionManager transactionManager, SecurityFoundation.Entities.UserProfileData entity, DeepSaveType deepSaveType, System.Type[] childTypes, DeepSession innerList)
		{	
			if (entity == null)
				return false;
							
			#region Composite Parent Properties
			//Save Source Composite Properties, however, don't call deep save on them.  
			//So they only get saved a single level deep.
			
			#region ApplicationKeySource
			if (CanDeepSave(entity, "ApplicationDefination|ApplicationKeySource", deepSaveType, innerList) 
				&& entity.ApplicationKeySource != null)
			{
				DataRepository.ApplicationDefinationProvider.Save(transactionManager, entity.ApplicationKeySource);
				entity.ApplicationKey = entity.ApplicationKeySource.ApplicationKey;
			}
			#endregion 
			#endregion Composite Parent Properties

			// Save Root Entity through Provider
			if (!entity.IsDeleted)
				this.Save(transactionManager, entity);
			
			//used to hold DeepSave method delegates and fire after all the local children have been saved.
			Dictionary<string, KeyValuePair<Delegate, object>> deepHandles = new Dictionary<string, KeyValuePair<Delegate, object>>();
			//Fire all DeepSave Items
			foreach(KeyValuePair<Delegate, object> pair in deepHandles.Values)
		    {
                pair.Key.DynamicInvoke((object[])pair.Value);
		    }
			
			// Save Root Entity through Provider, if not already saved in delete mode
			if (entity.IsDeleted)
				this.Save(transactionManager, entity);
				

			deepHandles = null;
						
			return true;
		}
		#endregion
	} // end class
	
	#region UserProfileDataChildEntityTypes
	
	///<summary>
	/// Enumeration used to expose the different child entity types 
	/// for child properties in <c>SecurityFoundation.Entities.UserProfileData</c>
	///</summary>
	public enum UserProfileDataChildEntityTypes
	{
		
		///<summary>
		/// Composite Property for <c>ApplicationDefination</c> at ApplicationKeySource
		///</summary>
		[ChildEntityType(typeof(ApplicationDefination))]
		ApplicationDefination,
		}
	
	#endregion UserProfileDataChildEntityTypes
	
	#region UserProfileDataFilterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="SqlFilterBuilder&lt;UserProfileDataColumn&gt;"/> class
	/// that is used exclusively with a <see cref="UserProfileData"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class UserProfileDataFilterBuilder : SqlFilterBuilder<UserProfileDataColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the UserProfileDataFilterBuilder class.
		/// </summary>
		public UserProfileDataFilterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the UserProfileDataFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public UserProfileDataFilterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the UserProfileDataFilterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public UserProfileDataFilterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion UserProfileDataFilterBuilder
	
	#region UserProfileDataParameterBuilder
	
	/// <summary>
	/// A strongly-typed instance of the <see cref="ParameterizedSqlFilterBuilder&lt;UserProfileDataColumn&gt;"/> class
	/// that is used exclusively with a <see cref="UserProfileData"/> object.
	/// </summary>
	[CLSCompliant(true)]
	public class UserProfileDataParameterBuilder : ParameterizedSqlFilterBuilder<UserProfileDataColumn>
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the UserProfileDataParameterBuilder class.
		/// </summary>
		public UserProfileDataParameterBuilder() : base() { }

		/// <summary>
		/// Initializes a new instance of the UserProfileDataParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		public UserProfileDataParameterBuilder(bool ignoreCase) : base(ignoreCase) { }

		/// <summary>
		/// Initializes a new instance of the UserProfileDataParameterBuilder class.
		/// </summary>
		/// <param name="ignoreCase">Specifies whether to create case-insensitive statements.</param>
		/// <param name="useAnd">Specifies whether to combine statements using AND or OR.</param>
		public UserProfileDataParameterBuilder(bool ignoreCase, bool useAnd) : base(ignoreCase, useAnd) { }

		#endregion Constructors
	}

	#endregion UserProfileDataParameterBuilder
	
	#region UserProfileDataSortBuilder
    
    /// <summary>
    /// A strongly-typed instance of the <see cref="SqlSortBuilder&lt;UserProfileDataColumn&gt;"/> class
	/// that is used exclusively with a <see cref="UserProfileData"/> object.
    /// </summary>
    [CLSCompliant(true)]
    public class UserProfileDataSortBuilder : SqlSortBuilder<UserProfileDataColumn>
    {
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the UserProfileDataSqlSortBuilder class.
		/// </summary>
		public UserProfileDataSortBuilder() : base() { }

		#endregion Constructors

    }    
    #endregion UserProfileDataSortBuilder
	
} // end namespace
